==> kapla-2.1.3.tgz: package/lib/core/Application.js <==
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

import exclamations from '../errors';
import { defaultSchema } from './schema';
import { Manager } from './Manager';
import { customEvents } from '../events';
export var Application =
/*#__PURE__*/
function () {
  _createClass(Application, null, [{
    key: "start",
    value: function start(element, schema, props) {
      var application = new Application(element, schema, props);
      application.start();
      return application;
    }
  }]);

  function Application() {
    var element = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : document.body;
    var schema = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : defaultSchema;
    var props = arguments.length > 2 ? arguments[2] : undefined;

    _classCallCheck(this, Application);

    this.element = element;
    this.schema = schema;
    this.props = props;
    this.manager = new Manager(this);
    this.customEvents = customEvents;
    this.plugins = new Set();
  }

  _createClass(Application, [{
    key: "start",
    value: function start() {
      this.manager.start();
    }
  }, {
    key: "stop",
    value: function stop() {
      this.manager.stop();
    }
  }, {
    key: "register",
    value: function register(slug, ComponentConstructor) {
      for (var _len = arguments.length, args = new Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
        args[_key - 2] = arguments[_key];
      }

      this.manager.addModule({
        slug: slug,
        ComponentConstructor: ComponentConstructor
      }, false, args);
    }
  }, {
    key: "load",
    value: function load(definitions) {
      var _this = this;

      var items = Array.isArray(definitions) ? definitions : [definitions];
      items.forEach(function (def) {
        return _this.manager.addModule(def);
      });
    }
  }, {
    key: "init",
    value: function init(slug, NoComponentConstructor) {
      for (var _len2 = arguments.length, args = new Array(_len2 > 2 ? _len2 - 2 : 0), _key2 = 2; _key2 < _len2; _key2++) {
        args[_key2 - 2] = arguments[_key2];
      }

      this.manager.addModule({
        slug: slug,
        ComponentConstructor: NoComponentConstructor
      }, true, args); // Return the instance

      return this.components.find(function (component) {
        return component instanceof NoComponentConstructor;
      });
    }
  }, {
    key: "unload",
    value: function unload(slugs) {
      var _this2 = this;

      var items = Array.isArray(slugs) ? slugs : [slugs];
      items.forEach(function (slug) {
        return _this2.manager.removeModule(slug);
      });
    }
    /**
     * Get "active" components
     *
     * @returns {Array} components
     * @readonly
     * @memberof Application
     */

  }, {
    key: "instanceByElement",

    /**
     * Get instance by element
     *
     * @param {HTMLElement} el component main element
     * @returns {Object} component instance
     * @memberof Application
     */
    value: function instanceByElement(el) {
      return this._getInstanceByElement(el);
    }
    /**
     * Get instance by element in async mode
     * more reliable implementation with MutationObserver async behaviour
     * use case : when component is added to the page
     * concurrently to the code execution
     *
     * @param {HTMLElement} el component main element
     * @returns {Object} component instance
     * @memberof Application
     */

  }, {
    key: "instanceByElementAsync",
    value: function instanceByElementAsync(el) {
      var _this3 = this;

      return this._nextTask().then(function () {
        return _this3._getInstanceByElement(el);
      });
    }
  }, {
    key: "_getInstanceByElement",
    value: function _getInstanceByElement(el) {
      return this.components.find(function (c) {
        return c.$el === el;
      });
    }
    /**
     * Get instances by component class
     *
     * @param {Function} Component component class
     * @returns {Object[]} component instances list
     * @memberof Application
     */

  }, {
    key: "instancesByComponent",
    value: function instancesByComponent(Component) {
      return this._getInstancesByComponent(Component);
    }
    /**
     * Get instances by component class in async mode
     * more reliable implementation with MutationObserver async behaviour
     * use case : when component is added to the page
     * concurrently to the code execution
     *
     * @param {Function} Component component class
     * @returns {Object[]} component instances list
     * @memberof Application
     */

  }, {
    key: "instancesByComponentAsync",
    value: function instancesByComponentAsync(Component) {
      var _this4 = this;

      // eslint-disable-line id-length
      return this._nextTask().then(function () {
        return _this4._getInstancesByComponent(Component);
      });
    }
  }, {
    key: "_getInstancesByComponent",
    value: function _getInstancesByComponent(Component) {
      return this.components.filter(function (c) {
        return c instanceof Component;
      });
    } // https://jakearchibald.com/2015/tasks-microtasks-queues-and-schedules/
    // eslint-disable-next-line class-methods-use-this

  }, {
    key: "_nextTask",
    value: function _nextTask() {
      return new Promise(function (resolve) {
        setTimeout(function () {
          resolve();
        }, 0);
      });
    }
  }, {
    key: "use",
    value: function use(type, event, log) {
      if (this.customEvents.types.has(type)) {
        this.handleError('oups', "This event type already exists [".concat(type, "]"));
      }

      this.customEvents.add(type, event, log);
    }
  }, {
    key: "extend",
    value: function extend(plugin) {
      plugin.init();
      this.plugins.add(plugin);
    } // eslint-disable-next-line class-methods-use-this

  }, {
    key: "handleError",
    value: function handleError(error, message) {
      // DEV
      // console.error('%s\n\n%o\n\n%o', message, error, detail);
      throw new Error("\uD83E\uDD26 ".concat(exclamations.random(), "! ").concat(message, " \n ").concat(error));
    }
  }, {
    key: "components",
    get: function get() {
      return this.manager.contexts.map(function (context) {
        return context.component;
      });
    }
  }, {
    key: "events",
    get: function get() {
      return this.customEvents.events;
    }
  }]);

  return Application;
}();

==> kapla-2.1.6.tgz: package/lib/core/Application.js <==
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

import exclamations from '../errors';
import { defaultSchema } from './schema';
import { Manager } from './Manager';
import { customEvents } from '../events';
export var Application = /*#__PURE__*/function () {
  function Application() {
    var element = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : document.body;
    var schema = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : defaultSchema;
    var props = arguments.length > 2 ? arguments[2] : undefined;

    _classCallCheck(this, Application);

    this.element = element;
    this.schema = schema;
    this.props = props;
    this.manager = new Manager(this);
    this.customEvents = customEvents;
    this.plugins = new Set();
  }

  _createClass(Application, [{
    key: "start",
    value: function start() {
      this.manager.start();
    }
  }, {
    key: "stop",
    value: function stop() {
      this.manager.stop();
    }
  }, {
    key: "register",
    value: function register(slug, ComponentConstructor) {
      for (var _len = arguments.length, args = new Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
        args[_key - 2] = arguments[_key];
      }

      this.manager.addModule({
        slug: slug,
        ComponentConstructor: ComponentConstructor
      }, false, args);
    }
  }, {
    key: "load",
    value: function load(definitions) {
      var _this = this;

      var items = Array.isArray(definitions) ? definitions : [definitions];
      items.forEach(function (def) {
        return _this.manager.addModule(def);
      });
    }
  }, {
    key: "init",
    value: function init(slug, NoComponentConstructor) {
      for (var _len2 = arguments.length, args = new Array(_len2 > 2 ? _len2 - 2 : 0), _key2 = 2; _key2 < _len2; _key2++) {
        args[_key2 - 2] = arguments[_key2];
      }

      this.manager.addModule({
        slug: slug,
        ComponentConstructor: NoComponentConstructor
      }, true, args); // Return the instance

      return this.components.find(function (component) {
        return component instanceof NoComponentConstructor;
      });
    }
  }, {
    key: "unload",
    value: function unload(slugs) {
      var _this2 = this;

      var items = Array.isArray(slugs) ? slugs : [slugs];
      items.forEach(function (slug) {
        return _this2.manager.removeModule(slug);
      });
    }
    /**
     * Get "active" components
     *
     * @returns {Array} components
     * @readonly
     * @memberof Application
     */

  }, {
    key: "components",
    get: function get() {
      return this.manager.contexts.map(function (context) {
        return context.component;
      });
    }
    /**
     * Get instance by element
     *
     * @param {HTMLElement} el component main element
     * @returns {Object} component instance
     * @memberof Application
     */

  }, {
    key: "instanceByElement",
    value: function instanceByElement(el) {
      return this._getInstanceByElement(el);
    }
    /**
     * Get instance by element in async mode
     * more reliable implementation with MutationObserver async behaviour
     * use case : when component is added to the page
     * concurrently to the code execution
     *
     * @param {HTMLElement} el component main element
     * @returns {Object} component instance
     * @memberof Application
     */

  }, {
    key: "instanceByElementAsync",
    value: function instanceByElementAsync(el) {
      var _this3 = this;

      return this._nextTask().then(function () {
        return _this3._getInstanceByElement(el);
      });
    }
  }, {
    key: "_getInstanceByElement",
    value: function _getInstanceByElement(el) {
      return this.components.find(function (c) {
        return c.$el === el;
      });
    }
    /**
     * Get instances by component class
     *
     * @param {Function} Component component class
     * @returns {Object[]} component instances list
     * @memberof Application
     */

  }, {
    key: "instancesByComponent",
    value: function instancesByComponent(Component) {
      return this._getInstancesByComponent(Component);
    }
    /**
     * Get instances by component class in async mode
     * more reliable implementation with MutationObserver async behaviour
     * use case : when component is added to the page
     * concurrently to the code execution
     *
     * @param {Function} Component component class
     * @returns {Object[]} component instances list
     * @memberof Application
     */

  }, {
    key: "instancesByComponentAsync",
    value: function instancesByComponentAsync(Component) {
      var _this4 = this;

      // eslint-disable-line id-length
      return this._nextTask().then(function () {
        return _this4._getInstancesByComponent(Component);
      });
    }
  }, {
    key: "_getInstancesByComponent",
    value: function _getInstancesByComponent(Component) {
      return this.components.filter(function (c) {
        return c instanceof Component;
      });
    } // https://jakearchibald.com/2015/tasks-microtasks-queues-and-schedules/
    // eslint-disable-next-line class-methods-use-this

  }, {
    key: "_nextTask",
    value: function _nextTask() {
      return new Promise(function (resolve) {
        setTimeout(function () {
          resolve();
        }, 0);
      });
    }
  }, {
    key: "use",
    value: function use(type, event, log) {
      if (this.customEvents.types.has(type)) {
        this.handleError('oups', "This event type already exists [".concat(type, "]"));
      }

      this.customEvents.add(type, event, log);
    }
  }, {
    key: "events",
    get: function get() {
      return this.customEvents.events;
    }
  }, {
    key: "extend",
    value: function extend(plugin) {
      plugin.init();
      this.plugins.add(plugin);
    } // eslint-disable-next-line class-methods-use-this

  }, {
    key: "handleError",
    value: function handleError(error, message) {
      // DEV
      // console.error('%s\n\n%o\n\n%o', message, error, detail);
      throw new Error("\uD83E\uDD26 ".concat(exclamations.random(), "! ").concat(message, " \n ").concat(error));
    }
  }], [{
    key: "start",
    value: function start(element, schema, props) {
      var application = new Application(element, schema, props);
      application.start();
      return application;
    }
  }]);

  return Application;
}();
